/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.core.impl;

import java.util.BitSet;
import java.util.List;
import com.silverwrist.venice.security.AuditRecord;
import com.silverwrist.venice.htmlcheck.HTMLChecker;
import com.silverwrist.venice.core.DataException;
import com.silverwrist.venice.core.SideBoxDescriptor;

public interface EngineBackend
{
  public static final int HTMLC_POST_BODY = 0;
  public static final int HTMLC_POST_PSEUD = 1;
  public static final int HTMLC_PREVIEW_BODY = 2;
  public static final int HTMLC_ESCAPE_BODY_PSEUD = 3;

  public static final int IP_POSTSPERPAGE = 0;
  public static final int IP_POSTSATTOP = 1;
  public static final int IP_MAXSEARCHRETURN = 2;
  public static final int IP_MAXSIGMEMBERDISPLAY = 3;
  public static final int IP_MAXCONFMEMBERDISPLAY = 4;
  public static final int IP_NUMFRONTPAGEPOSTS = 5;
  public static final int IP_NUMAUDITRECSPERPAGE = 6;
  public static final int IP_CREATESIGLVL = 7;
  public static final int IPC_NUM_PARAMS = 8;

  public abstract SimpleEmailer createEmailer();

  public abstract String getStockMessage(String key);

  public abstract int getNewConfirmationNumber();

  public abstract String getCountryNameForCode(String code);

  public abstract String getLanguageNameForCode(String code);

  public abstract SIGData getSIGDataObject(int sigid) throws DataException;

  public abstract void detachSIGDataObject(int sigid);

  public abstract BitSet getLockedFeaturesMask();

  public abstract List getSIGFeatureSet(BitSet enabled_features, int level, boolean read_privs);

  public abstract String getAppletForFeature(int code);

  public abstract boolean isValidInitialFeatureIndex(short ndx);

  public abstract BitSet getDefaultFeaturesMask();

  public abstract void registerNewSIG(SIGData sig);

  public abstract int getFeatureIndexBySymbol(String symbol);

  public abstract boolean canAccessFeature(String symbol, int level, boolean has_read);

  public abstract ConferenceData getConferenceDataObject(int confid) throws DataException;

  public abstract void detachConferenceDataObject(int confid);

  public abstract void saveAuditRecord(AuditRecord ar);

  public abstract void registerNewConference(ConferenceData conf);

  public abstract HTMLChecker createCheckerObject(int type);

  public abstract int getParamInt(int selector);

  public abstract void forceParamReload() throws DataException;

  public abstract SideBoxDescriptor getMasterSideBoxDescriptor(int id);

  public abstract void startPublish();

  public abstract void publishNew(PublishedMessageImpl pubmsg);

  public abstract void unpublish(long postid);

} // end interface EngineBackend
