/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.servlets.format;

import java.io.Writer;
import java.io.IOException;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.log4j.*;

public class BaseJSPData
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */
  protected static final String ATTR_NAME = "com.silverwrist.venice.BaseJSPData";

  private static Category logger = Category.getInstance(BaseJSPData.class.getName());

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private String location;                // location string to use for this page
  private VeniceContent content;          // the actual content
  private boolean login_links = true;     // show login links up top?

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  public BaseJSPData(String location, VeniceContent content)
  {
    this.location = location;
    this.content = content;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * External static operations
   *--------------------------------------------------------------------------------
   */

  public static BaseJSPData retrieve(ServletRequest request)
  {
    return (BaseJSPData)(request.getAttribute(ATTR_NAME));

  } // end retrieve

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  public void store(ServletRequest request)
  {
    request.setAttribute(ATTR_NAME,this);

  } // end store

  public String getTitle(RenderData rdat)
  {
    return content.getPageTitle(rdat);

  } // end getTitle

  public boolean locationSpecified()
  {
    return (location!=null);

  } // end if

  public String getLocation()
  {
    return location;

  } // end getLocation

  public boolean displayLoginLinks()
  {
    return login_links && (location!=null);

  } // end displayLoginLinks

  public void displayLoginLinks(boolean value)
  {
    login_links = value;

  } // end displayLoginLinks

  public void transfer(ServletContext ctxt, RenderData rdat) throws IOException, ServletException
  {
    rdat.storeBaseJSPData(this);
    RequestDispatcher dispatcher = ctxt.getRequestDispatcher(rdat.getFormatJSPPath("base.jsp"));
    rdat.forwardDispatch(dispatcher);

  } // end transfer

  public void renderContent(ServletContext ctxt, Writer out, RenderData rdat)
      throws IOException, ServletException
  {
    if (content==null)
    { // there is no content!
      new ErrorBox(null,"Internal Error: There is no content available",null).renderHere(out,rdat);
      return;

    } // end if

    if (content instanceof ContentRender)
    { // we have an old-style ContentRender
      ContentRender cr = (ContentRender)content;
      cr.renderHere(out,rdat);
      return;

    } // end if
    else if (content instanceof JSPRender)
    { // we have a JSP-rendered page - include it
      JSPRender jr = (JSPRender)content;
      rdat.storeJSPRender(jr);
      RequestDispatcher dispatcher = ctxt.getRequestDispatcher(rdat.getFormatJSPPath(jr.getTargetJSPName()));
      out.flush();
      rdat.flushOutput();  // make sure the stuff to be output first is output
      rdat.includeDispatch(dispatcher);
      rdat.flushOutput();  // now make sure the included page is properly flushed
      return;
      
    } // end if
    else // this is the fallback if we don't recognize the content
      new ErrorBox(null,"Internal Error: Content of invalid type",null).renderHere(out,rdat);

  } // end renderContent

} // end class BaseJSPData
